static char rcsid[] = "$Id$";

/******************************************************************************
 *
 * (c) Copyright 1992-1998 Scheduling Systems Incorporated
 * All rights reserved.
 *
 * These programs are supplied under a license.  They may not be used,
 * disclosed, and/or copied except as permitted under such license
 * agreement.  Any copy must contain the above copyright notice and
 * this notice.  Use, copying, and/or disclosure of the programs is
 * strictly prohibited unless otherwise provided in the license agreement.
 *
 ******************************************************************************
 *
 * $Author$  Created By: mmorin
 * $Date$  Creation: Tue Sep 14 07:41:59 1999
 * $Revision$
 * $State$
 *
 * Revisions by:
 *      Al Asber                        (aasber)
 *      Ahmet M. Buharali               (amb, ahmet)
 *      Jeff Donner                     (jgd)
 *      Saeed Honaryar                  (sh)
 *      Massimo M. Morin                (mmm, mmorin)
 *      John D. Pararas                 (jdp)
 *      David J. Picard                 (djp, dave)
 *      John Sharko                     (jsharko)
 *      Tom Toomey                      (ttoomey)
 *
 * $Log$
 *
 *****************************************************************************/

#include <stdio.h>
#include <time.h>
#include "List.h"

/*
 *--------------------------------------------------------------
 *
 * List::List --
 *
 *	Create the the empty list of data;
 *
 * Arguments:
 *	
 * Results:
 *	Returns .
 *
 *--------------------------------------------------------------
 */

List::List () {
  m_head = NULL;
  m_tail = NULL;
  m_nObjects = 0;
}

/*
 *--------------------------------------------------------------
 *
 * List::~List --
 *
 *	Destroy the list of data
 *
 * Arguments:
 *	
 * Results:
 *	Returns .
 *
 *--------------------------------------------------------------
 */

List::~List () {

  Node* p = NULL;
  while (m_head != NULL) {
    p = m_head;
    m_head = m_head->Next();
    delete p;
  }
  
}

/*
 *--------------------------------------------------------------
 *
 * List::Load --
 *
 *	Load the contents of infile into the handle list;
 *	It is pushed in, so it is in the reverse order!
 *
 * Arguments:
 *	ifstream infile: the file to read from
 * Results:
 *	Returns void.
 *
 *--------------------------------------------------------------
 */
void
List::Load(const char* fname) {
  
  char * localName;
  if (fname == NULL || strcmp(fname, "") == 0) {
    localName = strdup ("workspace.dat");
  } else {
    localName = strdup(fname);
  }

  ifstream infile(localName);

  // load it!
  char objectName[100] = "";
  char buffer[1024] = "";
  char mark[1024] = "";
  const char* data = NULL;
  Node* node = NULL;
  
  cout << "Loading database from " << localName << endl;
  int i = 0;

  while (!infile.eof()) {
    infile.getline(buffer,1024);

    if (strcmp(buffer, "") == 0 || 
	buffer[0] == '#' ) { 
       continue;
    }

    strcpy (objectName, &buffer[6]);

    sprintf (mark, "BEGIN %s", objectName);
     if (strcmp(mark, buffer) != 0) {
      cerr << "Unmatch marker:\n" << buffer
	   << "\nSkipping next class\n";
      continue;
    }
      
     node = new Node(objectName);
     node->SetPrev(m_tail);

     if (m_tail != NULL) {
       m_tail->SetNext(node);
     }
     
     m_tail = node;

     if (m_head == NULL) {
       m_head = m_tail;
     }

     node->Load(infile);
  
  }
  
  infile.close();
  cout << "Loading completed!" << endl;
  DumpStatistics(cout);
  free (localName);
}



/*
 *--------------------------------------------------------------
 *
 * List::Save --
 *
 *	Save the contents of the list into a file.
 *
 * Arguments:
 *	const char* fname: the file name of the file to save data into (default NULL)
 *	bool stat: if setted save the statistics of it into the file too (default false)
 * Results:
 *	Returns void.
 *
 *--------------------------------------------------------------
 */
void 
List::Save (const char* fname, bool stats) {
  char * localName;
  if (fname == NULL || strcmp(fname, "") == 0) {
    localName = strdup ("workspace.dat");
  } else {
    localName = strdup(fname);
  }

  ofstream outfile(localName);

  // save it!

  time_t t;
  t = time(NULL);
  
  outfile << "# SSI WORKSPACE" << endl
	  << "# " << endl
	  << "# (c) Copyright 1992-1999 Scheduling Systems Incorporated." << endl
	  << "# All rights reserved." << endl
	  << "# " << endl
	  << "# File: " << localName << endl
	  << "# Date: " << ctime(&t);
  if (stats) {
    outfile << "# Statistics: " << endl;
    DumpStatistics(outfile);
  }
  
  outfile << "#\n\n";

  Node* node = m_head;
  while (node != NULL) {
    node->Save(outfile);
    node = node->Next();
  }

  outfile << "# End WorkSpace\n";
  outfile.close();
   
  cout << "Workspace Saved in " << localName << endl;
  DumpStatistics(cout);
  free (localName);
}




/*
 *--------------------------------------------------------------
 *
 * List::RecordStatistics --
 *
 *	Collect information on the data recorded
 *
 * Arguments:
 *	StatisticsType &stats: the holder of the statistics
 * Results:
 *	Returns void.
 *
 *--------------------------------------------------------------
 */
void
List::RecordStatistics (StatisticsType &stats) {

  Node* node = m_head;
  int i=0;
  while (node != NULL) {
    
    stats.data[i][0] = node->GetNElement();
    stats.data[i][1] = node->GetDepth() + 10;
    strcpy (stats.names[i], node->GetName()); // this is the name
     node = node->Next();
     i++;
  }  
}

/*
 *--------------------------------------------------------------
 *
 * List::DumpStatistics --
 *
 *	Puts the statistic on the proper fstream
 *
 * Arguments:
 *	ostream &outfile: where to show the info
 * Results:
 *	Returns void.
 *
 *--------------------------------------------------------------
 */
void
List::DumpStatistics (ostream &outfile) {
    Node* node = m_head;
    while (node != NULL) {
      outfile << "# ";
      node->DumpStatistics(outfile);
      outfile << endl;
      node = node->Next();
    }
}


/* -------------------------------------------------------------------------- */
/* END */
