static char rcsid[] = "$Id$";

/******************************************************************************
 *
 * (c) Copyright 1992-1998 Scheduling Systems Incorporated
 * All rights reserved.
 *
 * These programs are supplied under a license.  They may not be used,
 * disclosed, and/or copied except as permitted under such license
 * agreement.  Any copy must contain the above copyright notice and
 * this notice.  Use, copying, and/or disclosure of the programs is
 * strictly prohibited unless otherwise provided in the license agreement.
 *
 ******************************************************************************
 *
 * $Author$  Created By: mmorin
 * $Date$  Creation: Tue Sep 14 07:13:04 1999
 * $Revision$
 * $State$
 *
 * Revisions by:
 *      Al Asber                        (aasber)
 *      Ahmet M. Buharali               (amb, ahmet)
 *      Jeff Donner                     (jgd)
 *      Saeed Honaryar                  (sh)
 *      Massimo M. Morin                (mmm, mmorin)
 *      John D. Pararas                 (jdp)
 *      David J. Picard                 (djp, dave)
 *      John Sharko                     (jsharko)
 *      Tom Toomey                      (ttoomey)
 *
 * $Log$
 *
 *****************************************************************************/
#include <stdio.h>
#include "Node.h"

/*
 *--------------------------------------------------------------
 *
 * Node::Node --
 *
 *	Create a node of the list
 *
 * Arguments:
 *	const char* name: the name/type of the data inserted
 * Results:
 *	Returns .
 *
 *--------------------------------------------------------------
 */

Node::Node (const char *name, int arity ) {
  
  m_prev = NULL;
  m_next = NULL;
  m_element = 0;
  m_objectName = strdup(name);  
}

/*
 *--------------------------------------------------------------
 *
 * Node::~Node --
 *
 *	Destructor
 *
 * Arguments:
 *	
 * Results:
 *	Returns .
 *
 *--------------------------------------------------------------
 */

Node::~Node () {
  free (m_objectName);
}

/*
 *--------------------------------------------------------------
 *
 * LoadList --
 *
 *	Load the contents of infile into the handle list
 *
 * Arguments:
 *	ifstream infile: the file to read from
 * Results:
 *	Returns void.
 *
 *--------------------------------------------------------------
 */
void
Node::Load (ifstream &infile) {

  char mark[128] = "";
//  char objectName[128];
  char buffer[1024] = "";
  char message[8192] = "";
  int len;

  sprintf (mark, "END %s", GetName());
  Information info;
  while (infile) {

    strcpy (message, "");
    len = 0;
    while (infile) {
      infile.getline(buffer,1024);      
       
      if (strcmp(buffer, mark) == 0) {
	return;
      }

      if (strcmp (buffer, "*") != 0) {
	len += 1 + strlen(buffer);
	if (len > 8192) {
	  cerr << "Too long message to deal with: skipping!\n";
	} else {
	  strcat (message, buffer);	 
	  strcat (message, "\n");
	}
      } else {
	break;
      }	
    } 
    info.SetInformation(message);
    //    m_tree.Insert(strdup(message));
    m_tree.Insert(info);
    m_element++;
    
  }

}

/*
 *--------------------------------------------------------------
 *
 * Node::Save --
 *
 *	Save the contents of the node into a file
 *
 * Arguments:
 *	fstream &outfile: the file (stream) where to save the data
 * Results:
 *	Returns void.
 *
 *--------------------------------------------------------------
 */
void
Node::Save (ofstream &outfile) {
  outfile << "# Saving " << m_element << " objects\n";
  outfile << "BEGIN " << m_objectName << endl;

  m_tree.Save(outfile);
  
  outfile << "\nEND " << m_objectName << endl;
 
}


/* -------------------------------------------------------------------------- */
/* END */
