/* -*- mode:c++ -*- */

#ifndef NODE_DEFINITIONS_INCLUDED
#define NODE_DEFINITIONS_INCLUDED

#ifdef __cplusplus
const char NODE_rcsid[] = "$Id$";
#else
static const char NODE_rcsid[] = "$Id$";
#endif

/******************************************************************************
 *
 * (c) Copyright 1992-1998 Scheduling Systems Incorporated
 * All rights reserved.
 *
 * These programs are supplied under a license.  They may not be used,
 * disclosed, and/or copied except as permitted under such license
 * agreement.  Any copy must contain the above copyright notice and
 * this notice.  Use, copying, and/or disclosure of the programs is
 * strictly prohibited unless otherwise provided in the license agreement.
 *
 ******************************************************************************
 *
 * $Author$  Created By: mmorin
 * $Date$  Creation: Tue Sep 14 07:08:36 1999
 * $Revision$
 * $State$
 *
 * Revisions by:
 *      Al Asber                        (aasber)
 *      Ahmet M. Buharali               (amb, ahmet)
 *      Jeff Donner                     (jgd)
 *      Saeed Honaryar                  (sh)
 *      Massimo M. Morin                (mmorin)
 *      John D. Pararas                 (jdp)
 *      David J. Picard                 (djp, dave)
 *      John Sharko                     (jsharko)
 *      Tom Toomey                      (ttoomey)
 *
 * $Log$
 *
 *****************************************************************************/
#include <fstream.h>

#include "BTree.h"
#include "Information.h"

class Node {
private:
  char* m_objectName;
  //  BTree<char*> m_tree;
  BTree<Information> m_tree;
  int m_element;
  Node* m_prev;
  Node* m_next;
  
public:
  Node(const char *name, int arity = 3);
  ~Node();

  Node* Next() {return m_next; }
  Node* Prev() {return m_prev; }
  void SetNext(Node* p) { m_next = p; }
  void SetPrev(Node* p) { m_prev = p; }
  char* GetName() { return m_objectName; }
  
  int GetNElement() { return m_element; }
  int GetDepth() { return m_tree.GetHeight(); }

  void Save(ofstream &output);  
  void Load(ifstream &input);

  void DumpStatistics (ostream &os) {
    os << GetName() << " : "
       << GetNElement() << " elements, "
       << GetDepth() << " depth.";
  }
    
};

#endif /* NODE_DEFINITIONS_INCLUDED */
