static char rcsid[] = "$Id$";

/******************************************************************************
 *
 * (c) Copyright 1992-1998 Scheduling Systems Incorporated
 * All rights reserved.
 *
 * These programs are supplied under a license.  They may not be used,
 * disclosed, and/or copied except as permitted under such license
 * agreement.  Any copy must contain the above copyright notice and
 * this notice.  Use, copying, and/or disclosure of the programs is
 * strictly prohibited unless otherwise provided in the license agreement.
 *
 ******************************************************************************
 *
 * $Author$  Created By: mmorin
 * $Date$  Creation: Tue Sep 14 06:22:15 1999
 * $Revision$
 * $State$
 *
 * Revisions by:
 *      Al Asber                        (aasber)
 *      Ahmet M. Buharali               (amb, ahmet)
 *      Jeff Donner                     (jgd)
 *      Saeed Honaryar                  (sh)
 *      Massimo M. Morin                (mmm, mmorin)
 *      John D. Pararas                 (jdp)
 *      David J. Picard                 (djp, dave)
 *      John Sharko                     (jsharko)
 *      Tom Toomey                      (ttoomey)
 *
 * $Log$
 *
 *****************************************************************************/

#include "basics/sysconf.h"     /* should go before any other includes */

/*
 *--------------------------------------------------------------
 *
 * Tree::Tree --
 *
 *	Create an empty tree (it is a leaf)
 *
 * Arguments:
 *	
 * Results:
 *	Returns .
 *
 *--------------------------------------------------------------
 */

Tree::Tree () {
  m_info = NULL;
  m_left = NULL;
  m_right = NULL;
  m_father = NULL;
}

/*
 *--------------------------------------------------------------
 *
 * Tree::~Tree --
 *
 *	Destructor
 *
 * Arguments:
 *	
 * Results:
 *	Returns.
 *
 *--------------------------------------------------------------
 */

Tree::~Tree () {
  if (m_left != NULL) {
    delete m_left;
  }
  if (m_info != NULL) {
    free(m_info);
  }

  if (m_right != NULL) {
    delete m_right;
  }
  
}

/*
 *--------------------------------------------------------------
 *
 * Tree::Insert --
 *
 *	Insert the dat in the node, in the proper order
 *
 * Arguments:
 *	char* data: the data to insert
 * Results:
 *	Returns void.
 *
 *--------------------------------------------------------------
 */
void
Tree::Insert (char* data) {
  
  if (m_info == NULL) {
    m_data = strdup(data);
  }

  InsertNot
  if (strcmp(m_info, data) < 0) {    
    // go on the left
    if (m_left == NULL) {
      m_left = new Tree();
      m_left->SetFather(this);
    }    
    m_left->Insert(data);
  } else {
    // goes on the right
    if (m_right == NULL) {
      m_right = new Tree();
      m_right->SetFather(this);
    }
    m_right->Insert(data);
  }
  
}

/*
 *------------------------------------------------------------------------
 *
 *  BTreeNode::UpLeft --
 *
 *  This function is releated to the BTree.
 *  Rise the tree till the current node is a left son
 *  Be careful: the search in the BTree is done in the way it recursively
 *  reset the position to be the precedent one. Note so tha if we are 
 *  on the first position m_currPos will be -1, and that is handled 
 *  by the first if!
 *
 *------------------------------------------------------------------------
 */

Tree*
Tree::UpLeft() {  
  Tree *father = m_father;
  
  if (father == NULL) {
    return NULL;
  }
  
  int pos = father->m_currPos;

  if (pos == -1) {
      return father->UpLeft();
  } else if (father->Left(pos) == this) {
    if (pos == 0) {
      return father->UpLeft();
    } else {
      father->SetPos(pos-1);
      return father;
    }
  } else {    
    return father;
  }
}

/*
 *------------------------------------------------------------------------
 *
 *  BTree::Insert -- 
 *
 *  Insert an element into the BTree; It will start the recursion!
 *  This part will handle the root of the tree to be full;
 * 
 *  Parameter:
 *   element  (input by ref) is the element to insert
 *   ptr      (input/output) the node where to insert
 *
 *  Returned value:
 *   none
 *  Side effect:
 *   The insertion of the element on the head, and start recursion 
 *   (increment the height).
 *  Complexity: 
 *   O(th) 
 *
 *------------------------------------------------------------------------
 */
void
Tree::Insert (const char *element) {

  if (m_head->IsFull()) {
    m_height++;

    BTreeNode<TYPE> *newOne = new BTreeNode<TYPE>(2 * Arity() + 1);
    newOne->SetLeaf(false);
    newOne->Left(0) = m_head;
    
    SplitChild(newOne, 0, m_head);    
    
    m_head = newOne;
  }

  InsertInBTreeNonFullNode(element, m_head);

  m_nKeys++;
}

  
/* -------------------------------------------------------------------------- */
/* END */
