static char rcsid[] = "$Id$";

/******************************************************************************
 *
 * (c) Copyright 1992-1998 Scheduling Systems Incorporated
 * All rights reserved.
 *
 * These programs are supplied under a license.  They may not be used,
 * disclosed, and/or copied except as permitted under such license
 * agreement.  Any copy must contain the above copyright notice and
 * this notice.  Use, copying, and/or disclosure of the programs is
 * strictly prohibited unless otherwise provided in the license agreement.
 *
 ******************************************************************************
 *
 * $Author$  Created By: mmorin
 * $Date$  Creation: Tue Sep 14 05:14:02 1999
 * $Revision$
 * $State$
 *
 * Revisions by:
 *      Al Asber                        (aasber)
 *      Ahmet M. Buharali               (amb, ahmet)
 *      Jeff Donner                     (jgd)
 *      Saeed Honaryar                  (sh)
 *      Massimo M. Morin                (mmm, mmorin)
 *      John D. Pararas                 (jdp)
 *      David J. Picard                 (djp, dave)
 *      John Sharko                     (jsharko)
 *      Tom Toomey                      (ttoomey)
 *
 * $Log$
 *
 *****************************************************************************/

/*
 Problem: Sorting a Workspace
 
 Our application rely on a structure of a Workspace. This structure
 is a big database that is kept in memory by a server. A client
 connects to the server; the last will send part or all the
 workspace to the client. The server has the workspace alive via a connection to a
 database (sybase or oracle). The Server keep alive the client
 workspace via an transmission of up to date information.

 The communication between client and server is usually slow but it is
 fast enought for keeping the client up to date.
 
 The problem is the start up time that is very long (even a couple of
 hours [DB busy, and heavy traffic]) due to the initial exchange of
 the full workspace.

 This is ok on production system but on testing system waiting hours
 for having a server ready is inadmissible.
 
 Idea: let's save the workspace of a live server on file then work
 with that one.

 The problem is that the procedure in place for saving data on file
 are a little out of sync with the real classes (don't ask why :( ) so
 we have to check by hand if what we save and load from file is always
 the same information.

 Now, I'm able to save data on file, but I don't know how to compare 2
 files, even because the data is organized into subset of information
 and the data is not always sorted and stored in the same order (this
 mainly because the  data is saved in the workspace via a btree).

 The program: Well, we read the data and we order each subset. After
 that we  save it on disk again.

 At the end we print some useful info.
*/


#include <iostream.h>

#include "List.h"

// global variable
bool verbose;
bool silent;
char inFileName[128];
char outFileName[128];


void SetCmdLineOptions (int n, char *arg[]) {
  int i, j;

  i = 1;
  while (i<n) {
    if (arg[i][0] != '-') {
      cerr << arg[i] << "\tUnrecorgnize parameter\n";
      i++;
      continue;
    }
    switch (arg[i][1]) {
      case '-' : {
	if (strcmp (arg[i], "--help") == 0)
	  strcpy (arg[i], "-h");
	else if (strcmp (arg[i], "--verbose") == 0)
	  strcpy (arg[i], "-v");
	else if (strcmp (arg[i], "--silent") == 0)
	  strcpy (arg[i], "-s");
	else if (strcmp (arg[i], "--input") == 0)
	  strcpy (arg[i], "-i");
	else if (strcmp (arg[i], "--output") == 0)
	  strcpy (arg[i], "-o");
	else {
	  cerr << arg[i] <<  "\tUnrecorgnize option\n";
	  i+=2;
	}
	i--;
	break;
      }
      case 'h':
      case '?': {
	cout <<	arg[0] << "\nHELP:\n"
		"This program accept data from a file and show "
		"his output on anothers files.\n"
		"Other options are: \n"
		" -h -? --help\t\t show this information\n"
		" -v --verbose \t\t showing result on video\n"
		" -s --silent\t\t running silently \n"
		" -i --input <infile>\t specifie the input file;\n"
		 "\t\t\t default is workspace.dat\n"
		" -o --output <outfile>\t specifies the output file;\n"
		 "\t\t\t default is workspace.dat\n"
		"for more detail refered to relative "
		"documentation\n\nProgram Terminated!\n";
	exit (0);
	break;
      }
      case 'v': {
	verbose = true;
	/* set verbose mode active */
	break;
      }
      case 's':	{
	silent = true;
	/* set silent mode active */
	break;
      }
      case 'i': {
	i++;
	strcpy (inFileName, arg[i]);
	break;
      }
      case 'o': {
	i++;
	strcpy (outFileName, arg[i]);
	break;
      }
      default: {
	cerr << arg[i] << "\tUnrecorgnize option!\n";
	break;
      }
    }
    i++;
  } /* end-while */
}

void main (int argc,
	   char* argv[]) {

  List data;
//  char* inFileName = NULL;
//  char* outFileName = NULL;
  
  SetCmdLineOptions(argc, argv);

  
  data.Load(inFileName);
  StatisticsType  stats;

  data.RecordStatistics(stats);
  
  data.Save(outFileName, true);

  return 0;
}
  
